import java.awt.event.*;
import javax.swing.*;

import java.util.TreeSet;

public class CCGUI extends JFrame implements ActionListener{
	JMenuBar menuBar;
	JMenu fileMenu, convertMenu, skillsMenu;
	JMenuItem newItem, exitItem, convertItem;
	JRadioButtonMenuItem highestItem,cumulateItem;
	ButtonGroup skillGroup;
	BasicGUI basicInput;
	OutputGUI output;
	SkillsGUI skillsInput;
	FeatsGUI featsInput;
	ForceSkillsGUI forceSkillsInput;
	ForceFeatsGUI forceFeatsInput;
	JTabbedPane tabPane;

	public CCGUI(){
		super("D20 to D6 Character Converter");
		setSize(700,700);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
		tabPane=new JTabbedPane();
		
		basicInput = new BasicGUI();
		tabPane.addTab("Basic Information",null,basicInput);
		
		skillsInput=new SkillsGUI();
		tabPane.addTab("Skills",null,skillsInput);
		
		featsInput=new FeatsGUI();
		tabPane.addTab("Feats",null,featsInput);
		
		forceSkillsInput = new ForceSkillsGUI();
		tabPane.addTab("Force Skills", null, forceSkillsInput);
		
		forceFeatsInput = new ForceFeatsGUI();
		tabPane.addTab("Force Feats", null, forceFeatsInput);
		
		output = new OutputGUI();
		tabPane.addTab("Output",null,output);
		
		makeMenuBar();
		
		add(tabPane);
	}
	
	public void makeMenuBar(){
		menuBar=new JMenuBar();
		
		fileMenu=new JMenu("File");
		
		newItem=new JMenuItem("New");
		fileMenu.add(newItem);
		newItem.addActionListener(this);
		
		exitItem=new JMenuItem("Exit");
		fileMenu.add(exitItem);
		exitItem.addActionListener(this);
		
		menuBar.add(fileMenu);
		
		convertMenu=new JMenu("Convert");
		
		convertItem=new JMenuItem("Convert");
		convertMenu.add(convertItem);
		convertItem.addActionListener(this);
		
		menuBar.add(convertMenu);
		
		skillsMenu=new JMenu("Skill Add?");
		skillGroup=new ButtonGroup();
		
		highestItem=new JRadioButtonMenuItem("Use the highest");
		highestItem.setSelected(true);
		skillGroup.add(highestItem);
		skillsMenu.add(highestItem);
		
		cumulateItem=new JRadioButtonMenuItem("+2/other item");
		skillGroup.add(cumulateItem);
		skillsMenu.add(cumulateItem);
		
		menuBar.add(skillsMenu);
		
		setJMenuBar(menuBar);
	}
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource()==exitItem){
			System.exit(0);
		}
		if(e.getSource()==newItem){
			basicInput.clear();
			output.clear();
			skillsInput.clear();
			featsInput.clear();
			forceSkillsInput.clear();
			forceFeatsInput.clear();
		}
		if(e.getSource()==convertItem){
			write();
		}
	}
	
	public void write(){
		output.clear();
		Ability[] D6Stats = new Ability[6];
		String langsFluent = "";
		String specialAbilities = "";
		
		int[] forceTypes = null;
		TreeSet <String> [] forcePowers = new TreeSet[7]; 
		
		int defense = 0;
		int melee = 0;
		int ranged = 0;
		int speed = 0;
		
		//D6  Order: str dex con int wis cha
		//D20 Order: dex str per kno mec tec
		
		int[] D20Stats;
		
		int[] D20Modifiers = new int[6];
		
		
		try{
			//converting ability scores
			D20Stats = basicInput.getAbilityScores();
			
			convertAbilityScores(D20Stats, D6Stats);
			
			for(int i = 0; i < D20Modifiers.length; i++)
				D20Modifiers[i] = Utility.getD20Modifier(D20Stats[i]);
	
			//converting other values
			defense=basicInput.getDefense();
			D6Stats[0].addSkill("Dodge",defense,"",false);
			melee=basicInput.getMelee();
			ranged=basicInput.getRanged();
			speed=basicInput.getSpeed();
		}
		catch(Exception e){
			JOptionPane.showMessageDialog(this, "All of the fields (except name) on the basic information tab must be integers.");
		}
		
		//converting skills
		langsFluent = convertSkills(D6Stats, D20Modifiers, langsFluent);
		
		//converting feats
		specialAbilities = convertFeats(D6Stats, forceTypes, specialAbilities, melee, ranged);
		
		if(specialAbilities.contains("Force Sensitive")){
			forceTypes = new int[3];
			forceTypes[0] = 3;
			forceTypes[1] = 3;
			forceTypes[2] = 3;
		}
		
		if(forceTypes != null){
			System.out.println("IN");
			convertForceSkills(forceTypes, forcePowers, D20Modifiers);
			
			convertForceFeats(forceTypes, forcePowers);
		}
		
		//printing
		output.println("Name: "+ basicInput.getName()+"\n");
		
		for(int i = 0; i < D6Stats.length; i++)
			output.print(D6Stats[i]);
		
		if(forceTypes != null){
			output.print("\nControl: " + Utility.toD6(forceTypes[0]));
			output.print("\nSense: " + Utility.toD6(forceTypes[1]));
			output.print("\nAlter: " + Utility.toD6(forceTypes[2]));
			
			output.print("\nControl Powers:" + forcePowers[0]);
			output.print("\nSense Powers:" + forcePowers[1]);
			output.print("\nAlter Powers:" + forcePowers[2]);
			output.print("\nControl and Sense Powers:" + forcePowers[3]);
			output.print("\nControl and Alter Powers:" + forcePowers[4]);
			output.print("\nSense and Alter Powers:" + forcePowers[5]);
			output.print("\nControl, Sense, and Alter Powers:" + forcePowers[6]);
		}
			
		output.println("\nSpeed:"+speed);
		output.println("\nLanguages Fluent:"+langsFluent);
		output.println("\nSpecial Abilities:"+specialAbilities);
	}
	
	private void convertAbilityScores(int[] D20Stats, Ability[] D6Stats){
		
		
		int temp=D20Stats[1];
		D6Stats[0]=new Ability("Dexterity",Utility.valueMatch(temp),this);
		
		temp=(int)Math.ceil((D20Stats[0]+D20Stats[2])/2.0);
		D6Stats[1]=new Ability("Strength",Utility.valueMatch(temp),this);
		
		temp=(int)Math.ceil((D20Stats[3]+D20Stats[4]+D20Stats[5])/3.0);
		D6Stats[2]=new Ability("Perception",Utility.valueMatch(temp),this);
		
		temp=(int)Math.ceil((D20Stats[3]+D20Stats[4])/2.0);
		D6Stats[3]=new Ability("Knowledge",Utility.valueMatch(temp),this);
		
		temp=(int)Math.ceil((D20Stats[1]+D20Stats[3])/2.0);
		D6Stats[4]=new Ability("Mechanical",Utility.valueMatch(temp),this);
		
		temp=D20Stats[3];
		D6Stats[5]=new Ability("Technical",Utility.valueMatch(temp),this);
	}
	
	private String convertSkills(Ability[] D6Stats, int[] D20Modifiers, String langsFluent){
		String[] skills = skillsInput.getSkills();
		int[] attributes = skillsInput.getSkillAttributes();
		String[] skillNotes = skillsInput.getSkillNotes();
		String[] skillTargets = skillsInput.getSkillTargets();
		int[] skillRanks = skillsInput.getSkillRanks();		
		boolean remove = skillsInput.isRemoveModifier();
		
		for(int i = 0; i < skills.length; i++){
			try{
				String skillName = skills[i];
				String skillNote = skillNotes[i];
				int ranks = skillRanks[i];
				if(remove && attributes[i] != 6){
					ranks -= D20Modifiers[attributes[i]];
					ranks = Math.max(ranks, 0);
				}
				
				String[] skillTarget = skillTargets[i].split(",");
				for(int j = 0; j < skillTarget.length; j += 2){
					char ability = skillTarget[j].charAt(0);
					String currTarget = skillTarget[j + 1];
					if(skillName.equals("Read/Write") || skillName.equals("Speak")){
						if(!langsFluent.contains(skillNote)){
							D6Stats[Utility.abilityMatch(ability)].addSkill(currTarget,2,"",true);
							langsFluent += skillNote+", ";}
					}else{
						D6Stats[Utility.abilityMatch(ability)].addSkill(currTarget,ranks,(skillNote==null)?"":skillNote,true);	
					}
				}
			}
			catch(Exception e){
				JOptionPane.showMessageDialog(this, "There was an error on the skills page with the skill:" + skills[i]);
			}
				
		}
		return langsFluent;
	}
	
	private String convertFeats(Ability[] D6Stats, int[] forceTypes, String specialAbilities, int melee, int ranged){
		String[] feats = featsInput.getFeats();
		String[] featTargets = featsInput.getFeatTargets();
		//String[] featNotes=featsInput.getFeatNotes(); Never gets used right now
				
		for(int i = 0; i < feats.length; i++){
			String featName = feats[i];
			try{
				if( featName.equals("Force Sensitive")){
					specialAbilities += "Force Sensitive, ";
				}else{
					//String featNote=featNotes[x]; Ditto
					String[] featTarget = featTargets[i].split(",");
					if(!featTarget[0].equals("Nothing")){
						if(featTarget[0].equals("O"))
							specialAbilities += featTarget[1]+", ";
						else
							for(int j = 0; j < featTarget.length; j += 3){
								int bonus = 0;
								if(Character.isDigit(featTarget[j+2].charAt(0)))
									bonus = Integer.parseInt(featTarget[j+2]);
								if(featTarget[j+2].equals("R"))
									bonus = ranged;
								if(featTarget[j+2].equals("M"))
									bonus = melee;
								D6Stats[Utility.abilityMatch(featTarget[j].charAt(0))].addSkill(featTarget[j+1],bonus,"",false);		
							}
					}	
				}
			}
			catch(Exception e){
				JOptionPane.showMessageDialog(this, "There is an error on the feats tab on feat: " + featName);
			}
		}
		return specialAbilities;
	}
	
	private void convertForceSkills(int[] forceTypes, TreeSet <String> [] forcePowers, int[] D20Modifiers){
		for(int i = 0 ; i < forcePowers.length; i++){
			forcePowers[i] = new TreeSet <String> ();
		}
		
		String[] skills = forceSkillsInput.getSkills();
		int[] attributes = forceSkillsInput.getSkillAttributes();
		//String[] skillNotes = forceSkillsInput.getSkillNotes();
		String[] skillTargets = forceSkillsInput.getSkillTargets();
		int[] skillRanks = forceSkillsInput.getSkillRanks();	
		boolean remove = forceSkillsInput.isRemoveModifier();
		
		for(int i = 0; i < skillTargets.length; i++){
			String skillName = skills[i];
			try{
				//String skillNote = skillNotes[i];
				int ranks = skillRanks[i];
				if(remove && attributes[i] != 6){
					ranks -= D20Modifiers[attributes[i]];
					ranks = Math.max(ranks, 0);
				}
				
				int pips = (int) Math.ceil( ranks / 2.0 );
				
				String[] skillTarget = skillTargets[i].split(",");
				int type = Utility.forceTypeMatch(skillTarget[0]);
				
				for(int j = 1; j < skillTarget.length; j++){
					forcePowers[type].add(skillTarget[j]);
				}
				
				switch(type){
					case 0: case 1: case 2:
						forceTypes[type] += pips;
						break;
						
					case 3:
						forceTypes[0] += pips / 2;
						forceTypes[1] += pips / 2;
						if( pips % 2 == 1 )
							forceTypes[0] ++;
						break;
						
					case 4:
						forceTypes[0] += pips / 2;
						forceTypes[2] += pips / 2;
						if( pips % 2 == 1 )
							forceTypes[2] ++;
						break;
						
					case 5:
						forceTypes[1] += pips / 2;
						forceTypes[2] += pips / 2;
						if( pips % 2 == 1 )
							forceTypes[1]++;
						break;
						
					case 6:
						forceTypes[0] += pips / 3;
						forceTypes[1] += pips / 3;
						forceTypes[2] += pips / 3;
						if( pips % 3 > 0 )
							forceTypes[0]++;
						if( pips % 3 == 2)
							forceTypes[1]++;
						break;
				}
			}
			catch(Exception e){
				JOptionPane.showMessageDialog(this, "There was an error on the force skills page on skill:" + skillName);
			}
		}
	}
	
	private void convertForceFeats(int[] forceTypes, TreeSet <String> [] forcePowers){
		String[] feats = forceFeatsInput.getFeats();
		String[] featTargets = forceFeatsInput.getFeatTargets();
		//String[] featNotes=featsInput.getFeatNotes(); Never gets used right now
				
		for(int i = 0; i < feats.length; i++){
			String featName = feats[i];
			try{
				if(!featTargets[i].equals("Nothing")){
					String[] featTarget = featTargets[i].split(",");
					int type = Utility.forceTypeMatch(featTarget[0]);
					System.out.println(type);
					
					for(int j = 1; j < featTarget.length; j++){
						forcePowers[type].add(featTarget[j]);
					}
					
					if(! (featName.equals("Control") || featName.equals("Sense") || featName.equals("Alter"))){
						switch(type){
							case 0: case 1: case 2:
								forceTypes[type] += 2;
								break;
								
							case 3:
								forceTypes[0] ++;
								forceTypes[1] ++;
								break;
								
							case 4:
								forceTypes[0] ++;
								forceTypes[2] ++;
								break;
								
							case 5:
								forceTypes[1] ++;
								forceTypes[2] ++;
								break;
								
							case 6:
								forceTypes[0] ++;
								forceTypes[1] ++;
								forceTypes[2] ++;
								break;
						}
					}
				}
			}catch(Exception e){
				JOptionPane.showMessageDialog(this, "There is an error on the force feats tab with feat:" + featName);
			}
		}
	}
	
	public boolean isHighestSkill(){
		return highestItem.isSelected();
	}
	
	public static void main(String[] args){
		new CCGUI().setVisible(true);
	}
}