import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

public class FeatsGUI extends JPanel implements ActionListener{
	
	JPanel featPanel,featTargetPanel,featNotePanel,top,mid;
	JButton addFeat;
	ArrayList<JComboBox> feats, featTargets;
	ArrayList<JTextField> featNotes;
	String[][] featConverts;
	String[] featList;
	
	public FeatsGUI(){
		super();
		
		feats = new ArrayList<JComboBox>();
		featTargets = new ArrayList<JComboBox>();
		featNotes = new ArrayList<JTextField>();
		
		readFeats();
		
		featList = new String[featConverts.length + 1];
		featList[0] = "";
		for(int i = 0; i < featConverts.length; i++){
			featList[i + 1] = featConverts[i][0];
		}
		Arrays.sort(featList);
		
		build();
	}
	
	
	void readFeats(){
		try{
			Scanner in = new Scanner( new File("Feats.txt") );
			int lines = Integer.parseInt(in.nextLine());
			featConverts = new String[lines][2];
			for(int i = 0; i < featConverts.length; i++){
				String temp = in.nextLine();
				int comma = temp.indexOf(',');
				featConverts[i][0] = temp.substring(0, comma);
				featConverts[i][1] = temp.substring(comma + 1);
			}	
		}catch(Exception e){}
	}
	
	private void build(){
		
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		
		top = new JPanel();
		addFeat = new JButton("Add Feat");
		addFeat.addActionListener(this);
		top.add(addFeat);
		
		mid = new JPanel();
		mid.setLayout(new BoxLayout(mid,BoxLayout.X_AXIS));
		
		featPanel = new JPanel();
		featPanel.setLayout(new BoxLayout(featPanel,BoxLayout.Y_AXIS));
		featPanel.add(new JLabel("Feat"));
		
		featTargetPanel = new JPanel();
		featTargetPanel.setLayout(new BoxLayout(featTargetPanel,BoxLayout.Y_AXIS));
		featTargetPanel.add(new JLabel("Skill Target"));
		
		featNotePanel = new JPanel();
		featNotePanel.setLayout(new BoxLayout(featNotePanel,BoxLayout.Y_AXIS));
		featNotePanel.add(new JLabel("Notes"));
		
		for(int i=0; i<5; i++)
		addFeatSlot();
		
		mid.add(featPanel);
		mid.add(featTargetPanel);
		mid.add(featNotePanel);
		
		add(top);
		add(mid);
		
	}
	
	public void clear(){
		removeAll();
		build();
	}
	
	private void addFeatSlot(){
		try{
			feats.add(new JComboBox(featList));
		}catch(Exception e){}
		feats.get(feats.size()-1).addActionListener(this);
		featPanel.add(feats.get(feats.size()-1));
		
		featTargets.add(new JComboBox());
		featTargets.get(featTargets.size()-1).setEditable(true);
		featTargetPanel.add(featTargets.get(featTargets.size()-1));
		
		featNotes.add(new JTextField());
		featNotePanel.add(featNotes.get(featNotes.size()-1));
		
		revalidate();
	}
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource() == addFeat){
			addFeatSlot();
		}
		if(e.getSource() instanceof JComboBox){
			int num = 0;
			for(int i = 0; i < feats.size(); i++){
				if(feats.get(i) == e.getSource()){
					num = i;
					break;
				}
			}
			featTargets.get(num).removeAllItems();
			featNotes.get(num).setText(null);
			
			String temp = feats.get(num).getSelectedItem().toString();
			
			if(temp.equals(""))
				return;
			
			String conversion = null;
			for(int i = 0; i < featConverts.length; i ++){
				if( featConverts[i][0].equals(temp) )
					conversion = featConverts[i][1];
			}
			if(conversion!=null){
				String[] D6Feat=conversion.split("[{}]+");
				for(int i = 1; i < D6Feat.length; i++){
					featTargets.get(num).addItem(D6Feat[i]);
				}
			}
		}
	}
		
	public String[] getFeats(){
		ArrayList<String> temp=new ArrayList<String>();
		
		for(int i = 0; i < feats.size(); i++){
			if(!feats.get(i).getSelectedItem().toString().equals(""))
				temp.add(feats.get(i).getSelectedItem().toString());
		}
		String[] tempReturn = new String[temp.size()];
		
		for(int i = 0; i < tempReturn.length; i++){
			tempReturn[i] = temp.get(i);
		}
		
		return tempReturn;
	}
	
	public String[] getFeatTargets(){
		ArrayList<String> temp=new ArrayList<String>();
		
		for(int i = 0; i < feats.size(); i++){
			if(!feats.get(i).getSelectedItem().toString().equals(""))
				temp.add(featTargets.get(i).getSelectedItem().toString());
		}
		System.out.println(temp.size());
		String[] tempReturn = new String[temp.size()];
		
		for(int i = 0; i < tempReturn.length; i++){
			tempReturn[i] = temp.get(i);
		}
		
		return tempReturn;
	}

	public String[] getFeatNotes(){
		ArrayList<String> temp=new ArrayList<String>();
		
		for(int i = 0; i < feats.size(); i++){
			if(!feats.get(i).getSelectedItem().toString().equals(""))
				temp.add(featNotes.get(i).getText());
		}
		System.out.println(temp.size());
		String[] tempReturn = new String[temp.size()];
		
		for(int i = 0; i < tempReturn.length; i++){
			tempReturn[i] = temp.get(i);
		}
		
		return tempReturn;
	}
}