import java.awt.event.*;
import javax.swing.*;

import java.util.*;
import java.io.*;

public class SkillsGUI extends JPanel implements ActionListener{
	
	JPanel skillPanel, attributePanel, skillTargetPanel, skillNotePanel, skillRankPanel, top, mid;
	JButton addSkill;
	JCheckBox modifier;
	ArrayList<JComboBox> skills;
	ArrayList<JComboBox> attributes;
	ArrayList<JComboBox> skillTargets;
	ArrayList<JTextField> skillNotes;
	ArrayList<JTextField> skillRanks;
	String[][] skillConverts;
	String[] skillList;
	String[] attributeList = {"Str","Dex","Con","Int","Wis","Cha","NA"};
	
	public SkillsGUI(){
		super();
		
		skills = new ArrayList<JComboBox>();
		attributes = new ArrayList<JComboBox>();
		skillTargets = new ArrayList<JComboBox>();
		skillNotes = new ArrayList<JTextField>();
		skillRanks = new ArrayList<JTextField>();
		
		readSkills();
		
		skillList = new String[skillConverts.length + 1];
		skillList[0] = "";
		for(int i = 0; i < skillConverts.length; i++){
			skillList[i + 1] = skillConverts[i][0];
		}
		Arrays.sort(skillList);
		
		build();
	}
	

	void readSkills(){
		try{
			Scanner in = new Scanner( new File("Skills.txt") );
			int lines = Integer.parseInt(in.nextLine());
			skillConverts = new String[lines][3];
			for(int i = 0; i < skillConverts.length; i++){
				String temp = in.nextLine();
				int comma1 = temp.indexOf(',');
				int comma2 = temp.indexOf(',', comma1 + 1);
				skillConverts[i][0] = temp.substring(0, comma1);
				skillConverts[i][1] = temp.substring(comma1 + 1, comma2);
				skillConverts[i][2] = temp.substring(comma2 + 1);
			}	
		}catch(Exception e){}
	}
	
	private void build(){
		
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		
		top = new JPanel();
		
		addSkill = new JButton("Add Skill");
		addSkill.addActionListener(this);
		top.add(addSkill);
		
		modifier = new JCheckBox("Remove D20 Modifiers from rank");
		top.add(modifier);
		
		mid = new JPanel();
		mid.setLayout(new BoxLayout(mid,BoxLayout.X_AXIS));
		
		skillPanel = new JPanel();
		skillPanel.setLayout(new BoxLayout(skillPanel,BoxLayout.Y_AXIS));
		skillPanel.add(new JLabel("Skill"));
		
		attributePanel = new JPanel();
		attributePanel.setLayout(new BoxLayout(attributePanel,BoxLayout.Y_AXIS));
		attributePanel.add(new JLabel("D20"));
		
		skillTargetPanel = new JPanel();
		skillTargetPanel.setLayout(new BoxLayout(skillTargetPanel,BoxLayout.Y_AXIS));
		skillTargetPanel.add(new JLabel("Target Skill"));
		
		skillNotePanel = new JPanel();
		skillNotePanel.setLayout(new BoxLayout(skillNotePanel,BoxLayout.Y_AXIS));
		skillNotePanel.add(new JLabel("Notes/Subskill"));
		
		skillRankPanel = new JPanel();
		skillRankPanel.setLayout(new BoxLayout(skillRankPanel,BoxLayout.Y_AXIS));
		skillRankPanel.add(new JLabel("Ranks"));
		
		for(int i = 0; i < 5; i++)
		addSkillSlot();
		
		mid.add(skillPanel);
		mid.add(attributePanel);
		mid.add(skillTargetPanel);
		mid.add(skillNotePanel);
		mid.add(skillRankPanel);
		
		add(top);
		add(mid);
	}
	
	public void clear(){
		removeAll();
		build();
	}
	
	private void addSkillSlot(){
		try{
			skills.add(new JComboBox(skillList));
		}catch(Exception e){}
		skills.get(skills.size()-1).addActionListener(this);
		skillPanel.add(skills.get(skills.size()-1));
		
		try{
			attributes.add(new JComboBox(attributeList));
		}catch(Exception e){}
		//attributes.get(attributes.size()-1).addActionListener(this);
		attributePanel.add(attributes.get(attributes.size()-1));
		
		skillTargets.add(new JComboBox());
		skillTargets.get(skillTargets.size()-1).setEditable(true);
		skillTargetPanel.add(skillTargets.get(skillTargets.size()-1));
		
		skillNotes.add(new JTextField());
		skillNotes.get(skillNotes.size()-1).setEditable(true);
		skillNotePanel.add(skillNotes.get(skillNotes.size()-1));
		
		skillRanks.add(new JTextField());
		skillRanks.get(skillRanks.size()-1).setColumns(2);
		skillRankPanel.add(skillRanks.get(skillRanks.size()-1));
		
		revalidate();
	}
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource() == addSkill){
			addSkillSlot();
		}
		if(e.getSource() instanceof JComboBox){
			int num = -1;
			for(int i = 0; i < skills.size(); i++){
				if(skills.get(i) == e.getSource()){
					num=i;
					break;
				}
			}
			
			skillTargets.get(num).removeAllItems();
			skillNotes.get(num).setText(null);
			skillRanks.get(num).setText(null);
			
			String temp = skills.get(num).getSelectedItem().toString();
			
			if(temp.equals(""))
				return;
			
			if(temp.equals("Read/Write")||temp.equals("Speak")){
				skillNotes.get(num).setText("Please Type in a Language");
			}
			
			String targetConversion = null;
			String D20Attribute = null;
			for(int i = 0; i < skillConverts.length; i ++){
				if( skillConverts[i][0].equals(temp) ){
					D20Attribute = skillConverts[i][1];
					targetConversion = skillConverts[i][2];
				}
			}
			
			if(targetConversion != null){
				for(int i = 0; i < attributeList.length; i++){
					if( D20Attribute.equals(attributeList[i]) ){
						attributes.get(num).setSelectedIndex(i);
					}
				}
				
				String[] D6Skill = targetConversion.split("[{}]+");
				for(int i = 1; i < D6Skill.length; i++){ //start at 1 to skip the empty string
					if(D6Skill[i].split(",")[0].equals("O")){
						skillTargets.get(num).addItem("N/A;You pick");
					}
					else
						skillTargets.get(num).addItem(D6Skill[i]);
				}
			}
		}
	}
	
	public String[] getSkills(){
		ArrayList<String> temp = new ArrayList<String>();
		
		for(int i = 0; i < skills.size(); i++){
			if(!skills.get(i).getSelectedItem().toString().equals(""))
				temp.add(skills.get(i).getSelectedItem().toString());
		}
		System.out.println(temp.size());
		String[] tempReturn = new String[temp.size()];
		
		for(int i = 0; i < tempReturn.length; i++){
			tempReturn[i]=temp.get(i);
		}
		
		return tempReturn;
	}
	
	public int[] getSkillAttributes(){
		ArrayList<Integer> temp = new ArrayList<Integer>();
		
		for(int i = 0; i < skills.size(); i++){
			if(!skills.get(i).getSelectedItem().toString().equals(""))
				temp.add(attributes.get(i).getSelectedIndex());
		}
		
		int[] tempReturn = new int[temp.size()];
		
		for(int i=0; i < tempReturn.length; i++){
			tempReturn[i]=temp.get(i);
		}
		
		return tempReturn;
	}
	
	public String[] getSkillTargets(){
		ArrayList<String> temp = new ArrayList<String>();
		
		for(int i = 0; i < skills.size(); i++){
			if(!skills.get(i).getSelectedItem().toString().equals(""))
				temp.add(skillTargets.get(i).getSelectedItem().toString());
		}
		
		String[] tempReturn = new String[temp.size()];
		
		for(int i=0; i < tempReturn.length; i++){
			tempReturn[i]=temp.get(i);
		}
		
		return tempReturn;
	}

	public String[] getSkillNotes(){
		ArrayList<String> temp = new ArrayList<String>();
		
		for(int i = 0; i < skills.size(); i++){
			if(!skills.get(i).getSelectedItem().toString().equals("")){
				Object temps = skillNotes.get(i).getText();
				temp.add((temps == null) ? null : temps.toString());
			}
		}
		
		String[] tempReturn = new String[temp.size()];
		
		for(int i = 0; i < tempReturn.length; i++){
			tempReturn[i] = temp.get(i);
		}
		
		return tempReturn;
	}
	
	public int[] getSkillRanks(){
		ArrayList<Integer> temp = new ArrayList<Integer>();
		
		for(int i = 0; i < skills.size(); i++){
			if(!skills.get(i).getSelectedItem().toString().equals("")){
				String tempRank = skillRanks.get(i).getText().toString();
				if(!tempRank.equals("")){
					try{
						temp.add(Integer.parseInt(tempRank));
					}
					catch(Exception e){
						JOptionPane.showMessageDialog(this, "The ranks on the skill page must be integers");
					}
				}else{
					temp.add(0);
				}
			}
		}
		
		int[] tempReturn = new int[temp.size()];
		
		for(int i = 0; i<tempReturn.length; i++){
			tempReturn[i] = temp.get(i);
		}
		
		return tempReturn;
	}
	
	public boolean isRemoveModifier(){
		return modifier.isSelected();
	}

}